/* Created 20121207 FI 
Works with heaps, clustered index, any compression 
Requires : http://www.sqlservercentral.com/scripts/Partitioning/73324/ 
exec dbo.sp_truncate_partitions 'SWITCH', 'DIM_CONTRACT_SIM_SERIAL' , 'tmp_DIM_CONTRACT_SIM_SERIAL_ZZZ', '2013-01-01', '2013-02-02' 
exec dbo.sp_truncate_partitions 'COPYTABLE', 'DIM_CONTRACT_SIM_SERIAL' , 'tmp_DIM_CONTRACT_SIM_SERIAL_ZZZ', null,null
exec dbo.sp_truncate_partitions 'DELETE', 'DIM_CONTRACT_SIM_SERIAL' , 'tmp_DIM_CONTRACT_SIM_SERIAL_ZZZ', '2013-01-01', '2013-02-02' 

*/ 

CREATE PROC dbo.sp_truncate_partitions 
( 
    @operation_type varchar ( 10 ) , 
    @Source_table varchar ( 100 ) , 
    @Dest_table varchar ( 100 ) , 
    @pv_from varchar ( 50 ) , 
    @pv_to varchar ( 50 ) 
) 
AS 

DECLARE 
--@table_pattern varchar(100) = 'DIM_CONTRACT_SIM_SERIAL'
@table_name varchar ( 100 ) = '' 
, @ps_name varchar ( 100 ) = '' 
, @pafu varchar ( 100 ) = '' 
, @str varchar ( max ) = '' 
, @nstr nvarchar ( 4000 ) = '' 
, @pn int = - 99 
, @pa_col_name varchar ( 100 ) = '' 
, @min_inde int 
, @data_compr varchar ( 20 ) = 'nnnn' 
, @pn_from int , @pn_to int 

BEGIN 
    DECLARE table_cursor CURSOR FOR 
    SELECT 
    table_name , 
    ps_name , 
    pa_col_name , 
    min ( index_id ) min_inde , 
    substring ( min ( pafu ) , 1 , charindex ( '(' , min ( pafu ) , 1 ) ) pafu 
    FROM view_all_partitions_trg_prod WHERE table_name = @source_table -- 'GSM_CDR_SMMT_TRAFFIC_ACTUAL' 
    GROUP BY 
    table_name , 
    ps_name , 
    pa_col_name 
    ORDER BY 1 

    OPEN table_cursor 

    FETCH NEXT FROM table_cursor 
    INTO @table_name , @ps_name , @pa_col_name , @min_inde , @pafu 

    WHILE @@FETCH_STATUS = 0 
    BEGIN 

        SET @data_compr = 
        ( 
            SELECT TOP 1 data_compression_desc FROM sys.tables t 
            JOIN sys.partitions p ON t.object_id = p.object_id 
            WHERE t.name = @table_name 
            AND index_id IN ( 0 , 1 ) 
            AND partition_number = 1 
        ) 

        SET @str = ' create table dbo.' + @Dest_table + ' ( ' 

        SELECT -- c.column_id,
        @str = @str + 
        CASE 
            WHEN c.column_id = 1 THEN '' 
            ELSE ' , ' 
        END + 
        ' [' + c.name + '] ' + 
        ' [' + t.name + '] ' + 
        CASE 
            WHEN t.name LIKE '%char' THEN ' (' + convert ( varchar ( 10 ) , c.max_length ) + ')' 
            WHEN t.name IN ( 'numeric' , 'decimal' ) THEN ' (' + convert ( varchar ( 10 ) , c.precision ) + ',' + convert ( varchar ( 10 ) , c.scale ) + ') ' 
            ELSE '' 
        END + 
        CASE c.is_nullable 
            WHEN 1 THEN ' NULL ' 
            WHEN 0 THEN ' NOT NULL ' 
            ELSE ' ' 
        END 
        -- into #structure
        FROM sys.tables tb 
        JOIN sys.columns c ON tb.object_id = c.object_id 
        JOIN sys.types t ON c.system_type_id = t.system_type_id WHERE tb.name = @table_name 
        ORDER BY c.column_id 

        IF @min_inde = 0 
        SET @str = @str + ' ) on ' + @ps_name + ' ( ' + @pa_col_name + ' ) with (data_compression = ' + @data_compr + ') ' 
        ELSE 
        BEGIN 
            SET @str = @str + ', constraint PK_' + @dest_table + ' primary key clustered ( ' 

            SELECT @str = @str + co.name + ',' FROM sys.index_columns ic 
            JOIN sys.columns co ON ic.object_id = co.object_id 
            AND co.column_id = ic.column_id 
            WHERE ic.object_id = object_id ( @table_name ) 
            AND index_id = 1 
            ORDER BY index_column_id 

            SET @str = substring ( @str , 1 , len ( @str ) - 1 ) 
            SET @str = @str + ' ) on ' + @ps_name + ' ( ' + @pa_col_name + ' ) ) with (data_compression = ' + @data_compr + ') ' 

        END 
        PRINT ( @str ) 

        EXEC ( @str ) 

        IF @operation_type IN ( 'DELETE' , 'SWITCH' ) 
        BEGIN 
            -- Fill pn_from, pn_to using dynamic SQL  
            SET @nstr = ' select @pnf=' + @pafu + '''' + @pv_from + ''') , @pnt=' + @pafu + '''' + @pv_to + ''')  ' 
            PRINT ( @nstr ) 

            EXEC sp_executesql @nstr , N'@pnf int OUTPUT, @pnt int OUTPUT' , 
            @pnf = @pn_from OUTPUT , @pnt = @pn_to OUTPUT ; 
            -- select @pn_from , @pn_to

            DECLARE part_cursor CURSOR FOR 
            SELECT partition_number 
            --, upper_date 
            FROM view_all_partitions_trg_prod 
            WHERE table_name = @table_name 
            AND index_id < 2 
            AND partition_number BETWEEN @pn_from AND @pn_to 

            OPEN part_cursor 
            FETCH NEXT FROM part_cursor INTO @pn 
            WHILE @@FETCH_STATUS = 0 
            BEGIN 
                SET @str = 'alter table ' + @table_name + ' switch partition ' + cast ( @pn AS varchar ( 6 ) ) + 
                ' to ' + @Dest_table + ' partition ' + cast ( @pn AS varchar ( 6 ) ) 
                PRINT ( @str ) 

                EXEC ( @str ) 
                FETCH NEXT FROM part_cursor INTO @pn 
            END 
            CLOSE part_cursor 
            DEALLOCATE part_cursor 
        END 
        -- @operation_type in (  'DELETE' , 'SWITCH') 

        IF @operation_type = 'DELETE' 
        BEGIN 
            SET @str = ' drop table ' + @Dest_table + ' ' 
            PRINT ( @str ) 

            EXEC ( @str ) 
        END 
        -- @operation_type = 'DELETE'  

        FETCH NEXT FROM table_cursor 
        INTO @table_name , @ps_name , @pa_col_name , @min_inde , @pafu 
    END 
    CLOSE table_cursor 
    DEALLOCATE table_cursor 

END