BEGIN
 SET NOCOUNT ON

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#dbfileinfo')
   )
 BEGIN
  DROP TABLE #dbfileinfo
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#logsizestats')
   )
 BEGIN
  DROP TABLE #logsizestats
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#datafilestats')
   )
 BEGIN
  DROP TABLE #datafilestats
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#fixeddrives')
   )
 BEGIN
  DROP TABLE #fixeddrives
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#usados')
   )
 BEGIN
  DROP TABLE #usados
 END

 CREATE TABLE #fixeddrives (
  DriveLetter VARCHAR(10)
  ,MB_Free DEC(20, 2)
  )

 CREATE TABLE #datafilestats (
  DBName VARCHAR(255)
  ,DBId INT
  ,FileId TINYINT
  ,[FileGroup] TINYINT
  ,TotalExtents DEC(20, 2)
  ,UsedExtents DEC(20, 2)
  ,[Name] VARCHAR(255)
  ,[FileName] VARCHAR(400)
  )

 CREATE TABLE #logsizestats
  -- DBCC SQLPERF -- Provides statistics about how the transaction-log space was used in all databases. 
  -- It can also be used to reset wait and latch statistics.
  (
  DBName VARCHAR(255) NOT NULL PRIMARY KEY CLUSTERED
  ,-- Database Name -- Name of the database for the log statistics displayed.
  DBId INT
  ,LogFile REAL
  ,-- Log Size (MB) -- Actual amount of space available for the log. This amount is smaller 
  -- than the amount originally allocated for log space because the SQL Server 2005 
  -- Database Engine reserves a small amount of disk space for internal header information.
  LogFileUsed REAL
  ,-- Log Space Used (%) -- Percentage of the log file currently occupied with transaction log information.
  STATUS BIT
  ) -- Status -- Status of the log file. Always 0. 

 CREATE TABLE #dbfileinfo (
  [ServerName] VARCHAR(255)
  ,[DBName] VARCHAR(65)
  ,[LogicalFileName] VARCHAR(400)
  ,[UsageType] VARCHAR(30)
  ,[Size_MB] DEC(20, 2)
  ,[SpaceUsed_MB] DEC(20, 2)
  ,[MaxSize_MB] DEC(20, 2)
  ,[NextAllocation_MB] DEC(20, 2)
  ,[GrowthType] VARCHAR(65)
  ,[FileId] SMALLINT
  ,[GroupId] SMALLINT
  ,[PhysicalFileName] VARCHAR(400)
  ,[DateChecked] DATETIME
  )

 DECLARE @SQLString VARCHAR(3000)
 DECLARE @MinId INT
 DECLARE @MaxId INT
 DECLARE @DBName VARCHAR(255)
 DECLARE @tblDBName TABLE (
  RowId INT identity(1, 1)
  ,DBName VARCHAR(255)
  ,DBId INT
  )

 INSERT INTO @tblDBName (
  DBName
  ,DBId
  )
 SELECT [Name]
  ,DBId
 FROM master..sysdatabases
 WHERE (STATUS & 512) = 0 /*NOT IN (536,528,540,2584,1536,512,4194841)*/
 ORDER BY [Name]

 INSERT INTO #logsizestats (
  DBName
  ,LogFile
  ,LogFileUsed
  ,STATUS
  )
 EXEC ('dbcc sqlperf(logspace) with no_infomsgs')

 UPDATE #logsizestats
 SET DBId = db_id(DBName)

 INSERT INTO #fixeddrives
 EXEC master..xp_fixeddrives

 SELECT @MinId = min(RowId)
  ,@MaxId = max(RowId)
 FROM @tblDBName

 WHILE (@MinId <= @MaxId)
 BEGIN
  SELECT @DBName = [DBName]
  FROM @tblDBName
  WHERE RowId = @MinId

  SELECT @SQLString = 'SELECT ServerName = @@SERVERNAME,' + ' DBName = ''' + @DBName + ''',' + ' LogicalFileName = [name],' + ' UsageType = CASE WHEN (64&[status])=64 THEN ''Log'' ELSE ''Data'' END,' + ' Size_MB = [size]*8/1024.00,' + ' SpaceUsed_MB = NULL,' +
   -- 20081125 Arithmetic overflow error converting expression to data type int. 
   -- ' MaxSize_MB = CASE [maxsize] WHEN -1 THEN -1 WHEN 0 THEN [size]*8/1024.00 ELSE maxsize*8/1024.00 END,'+
   ' MaxSize_MB = CASE [maxsize] WHEN -1 THEN -1 WHEN 0 THEN [size]*8/1024.00 ELSE maxsize/1024.00*8 END,' +
   -- 20081125 end 
   ' NextExtent_MB = CASE WHEN (1048576&[status])=1048576 THEN ([growth]/100.00)*([size]*8/1024.00) WHEN [growth]=0 THEN 0 ELSE [growth]*8/1024.00 END,' + ' GrowthType = CASE WHEN (1048576&[status])=1048576 THEN ''%'' ELSE ''Pages'' END,' + ' FileId = [fileid],' + ' GroupId = [groupid],' + ' PhysicalFileName= [filename],' + ' CurTimeStamp = GETDATE()' +
   -- 20081125 begin @DBName embedded spaces 
   -- 'FROM '+@DBName+'..sysfiles'  
   'FROM [' + @DBName + ']..sysfiles'

  -- 20081125 end 
  PRINT @SQLString

  INSERT INTO #dbfileinfo
  EXEC (@SQLString)

  UPDATE #dbfileinfo
  -- 20081125 begin LogFileUsed is % 
  -- SET SpaceUsed_MB = (SELECT LogFileUsed FROM #LogSizeStats WHERE DBName = @DBName)
  SET SpaceUsed_MB = Size_MB / 100.0 * (
    SELECT LogFileUsed
    FROM #logsizestats
    WHERE DBName = @DBName
    )
  -- 20081125 end 
  WHERE UsageType = 'Log'
   AND DBName = @DBName

  -- 20081125 begin @DBName embedded spaces 
  -- SELECT @SQLString = 'USE ' + @DBName + ' DBCC SHOWFILESTATS WITH NO_INFOMSGS'
  SELECT @SQLString = 'USE [' + @DBName + '] DBCC SHOWFILESTATS WITH NO_INFOMSGS'

  -- 20081125 end 
  INSERT #datafilestats (
   FileId
   ,[FileGroup]
   ,TotalExtents
   ,UsedExtents
   ,[Name]
   ,[FileName]
   )
  EXECUTE (@SQLString)

  UPDATE #dbfileinfo
  SET [SpaceUsed_MB] = S.[UsedExtents] * 64 / 1024.00
  FROM #dbfileinfo AS F
  INNER JOIN #datafilestats AS S ON F.[FileId] = S.[FileId]
   AND F.[GroupId] = S.[FileGroup]
   AND F.[DBName] = @DBName

  TRUNCATE TABLE #datafilestats

  SELECT @MinId = @MinId + 1
 END

 SELECT @@servername AS servidor
  ,substring(A.PhysicalFileName, 1, 1) AS unidad
  ,sum([Size_MB]) AS SqlTotalDB
  ,sum([SpaceUsed_MB]) AS SqlTotalUsedSpaceDB
  ,sum(([Size_MB]) - ([SpaceUsed_MB])) AS SQLTotalFreeSpaceDB
 INTO #usados
 FROM #dbfileinfo AS A
 LEFT JOIN #fixeddrives AS B ON substring(A.PhysicalFileName, 1, 1) = B.DriveLetter
 --ORDER BY CAST([NextAllocation_MB] AS decimal(6,2))  
 --ORDER BY [FreeSpaceInDrive]  
 --where ([Size_MB] - [SpaceUsed_MB]) -  ([NextAllocation_MB]) < 0 
 --where (B.MB_Free) + (([Size_MB] - [SpaceUsed_MB]) -  ([NextAllocation_MB])) < 2000
 --where (B.MB_Free) + (([Size_MB] - [SpaceUsed_MB]) -  ([NextAllocation_MB])) <= 0
 --where CONVERT(sysname,DatabasePropertyEx([DBName],'Recovery')) = 'FULL' 
 --where [MaxSize_MB] <> 2097152.00 and [MaxSize_MB] <> -1.00 
 --where left ([PhysicalFileName],1) = 'C' 
 --where (CAST(([Size_MB] - [SpaceUsed_MB]) / [Size_MB] AS decimal(4,2))) > 0.5 and ([Size_MB]) > 5000 --mas de la mitad de espacio libre
 --where ([Size_MB]) > 100000 --TAMA?O DE LA BASE DE DATOS 
 --where [DBName] like '%content%' 
 WHERE [UsageType] = 'data'
 --where [NextAllocation_MB] < 5 and [SpaceUsed_MB] > 20 and [dbname] not in ('master','model','msdb','tempdb') -- bases de datos grandes con poco crecimiento
 --where [NextAllocation_MB] > 500 -- bases de datos con crecimiento desmesurado
 --where [Size_MB] - [SpaceUsed_MB] < 50 and [UsageType] = 'log' 
 GROUP BY substring(A.PhysicalFileName, 1, 1)

 SELECT sum(SQLTotalFreeSpaceDB)
 FROM #fixeddrives AS f
 INNER JOIN #usados AS z ON z.unidad = f.DriveLetter

 --where MB_Free < 5000			 
 --select * from #usados 
 --ORDER BY ([Size_MB] - [SpaceUsed_MB]) -  ([NextAllocation_MB]) 
 --ORDER BY CAST([NextAllocation_MB] AS decimal(6,2))  
 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#dbfileinfo')
   )
 BEGIN
  DROP TABLE #dbfileinfo
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#logsizestats')
   )
 BEGIN
  DROP TABLE #logsizestats
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#datafilestats')
   )
 BEGIN
  DROP TABLE #datafilestats
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#fixeddrives')
   )
 BEGIN
  DROP TABLE #fixeddrives
 END

 IF EXISTS (
   SELECT 1
   FROM tempdb..sysobjects
   WHERE [Id] = object_id('tempdb..#usados')
   )
 BEGIN
  DROP TABLE #usados
 END

 -- 20081125 begin SET NOCOUNT OFF 
 SET NOCOUNT OFF
  -- 20081125 end 
END
